/*
Copyright (c) 2023 B Tasker

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

    Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

    Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

    Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

window.seen = [];
window.searchtype = "";
window.resultlimit = 100

String.prototype.replaceAll = function(search, replacement) {
    var target = this;
    return target.split(search).join(replacement);
};


function loadfromURL(){
    if (window.location.search){
        qs = parseQSTypeString(window.location.search);
        if (qs["q"]){
            document.getElementById('searchterm').value = decodeURIComponent(qs['q'].replaceAll("+","%20"));
            window.searchtype = decodeURIComponent(qs['t']);
            document.getElementById('searchtypev').value = window.searchtype;
            triggersearch();
        }
    }
}


function changeType(val){
    document.getElementById('searchtypev').value = val;
    document.getElementById('searchform').submit();
}


function triggersearch(){
    term = document.getElementById('searchterm').value;
    
    if (term.length < 3){
        alert("Minimum search term length is 3");
        return;
    }
    
    window.seen = [];
    
    searchobj = {
     "term" : term
    }
    
    if (window.resultlimit){
        searchobj["limit"] = window.resultlimit
    }
    
    collapseHelp();
    
    if (window.searchtype != "image"){
        //searchobj["query"]["bool"]["must_not"] = [{"match": {"content_type":"image"}}];
        searchobj['type'] = "DOC";
        imgwidthcss = document.getElementById('imgwidth');
        if (imgwidthcss){
                imgwidthcss.parentNode.removeChild(imgwidthcss);
        }
        
    }else{
        searchobj['type'] = "IMAGE";
        imgwidthcss = document.getElementById('imgwidth');
        if (!imgwidthcss){
            css = document.createElement('style');
            css.id = "imgwidth";
            css.innerHTML = "#resultsbox { width: 100% }";
            document.body.appendChild(css);
        }
    }
    

    fetchPage("/search/",searchobj,writeResults,failedSearch);
    document.getElementById('resultsbox').innerHTML = '';
    document.getElementById('loading').style.display = 'block';
    //window.history.pushState('Results-' + term, 'Search Results for "' + term + '"', '?q=' + encodeURI(term));    
}


function fetchPage(url,jsonobj,callback,errcallback){
    var xmlhttp;
    if (window.XMLHttpRequest){
        // code for IE7+, Firefox, Chrome, Opera, Safari
        xmlhttp=new XMLHttpRequest();
    }else{
        // code for IE6, IE5 (why am I still supporting these? Die... Die.... Die....
        xmlhttp=new ActiveXObject("Microsoft.XMLHTTP");
    }

    xmlhttp.onreadystatechange=function(){
        if (xmlhttp.readyState==4){
            if(xmlhttp.status==200){
                callback(xmlhttp.responseText)
            }else{
                errcallback(xmlhttp.responseText)
            }
   
        } 
        
}

    xmlhttp.open("POST",url);
    xmlhttp.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
    xmlhttp.send(JSON.stringify(jsonobj));
}

function parseQSTypeString(qs){
    //https://snippets.bentasker.co.uk/page-1708051221-Querystring-parsing-Javascript.html
    // Strip any leading cruft
    qs = qs.replace(/\?/,'').replace(/^#/,'');
    
    var result = {};
    var splitele;
    
    // Break the pairs up
    var pairs = qs.split('&');
    
    // Iterate over and populate our dict
    for (var i=0; i<pairs.length; i++){
        splitele = pairs[i].split("=");
        
        // There may be a trailing ampersand. Do nothing
        if (splitele[0].length > 0){
            result[splitele[0]] = decodeURI(splitele[1]);
        }
    }
    return result;
}

function buildQSTypeString(data){
    // https://snippets.bentasker.co.uk/page-1708051221-Querystring-parsing-Javascript.html
    var qs = '';
    for (var key in data){
        qs += key + "=" + encodeURI(data[key]) + "&"
    }
    return qs
}


function writeResults(resultjson){
    console.log("Search worked! woot");
    resp = JSON.parse(resultjson);
    results = resp['results'];
    document.getElementById('loading').style.display = 'none';
    
    resultcont = document.getElementById('resultsbox');
    
    var e;
    count=0;
    rel_images = 0;
    for (var i=0;i<results.length;i++){
    
        if (window.seen.includes(results[i]['key'])){
            // This will make the search counter wrong, but we'll cross that bridge later
            continue;
        }
    
        if (window.searchtype != "image"){
            e = createResultRow(results[i]);
            
            if (count==2){
                d = document.createElement('div');
                d.id = "imagecrosslink";
                resultcont.appendChild(d);
                relatedImagessearch();
                rel_images = 1;
            }
            
        }else{
            e = createImageNail(results[i]);
        }
        
        
        resultcont.appendChild(e);
        window.seen.push(results[i]['key']);
        count++;
    }
    
    if (window.searchtype != "image" && rel_images == 0){
        d = document.createElement('div');
        d.id = "imagecrosslink";
        resultcont.appendChild(d);
        relatedImagessearch();        
    }
    
    
    if (window.resultlimit && results.length == window.resultlimit){
        document.getElementById('resultcount').innerHTML = results.length + " results (<a href='javascript:uncap();'>capped</a>)";
    }else{
        document.getElementById('resultcount').innerHTML = results.length + " results";
    }    

    document.getElementById('resultinfo').style.display = 'block';
    
}

function uncap(){
   // Disable the results cap and resubmit the search
    window.resultlimit = false;
    triggersearch();
    
}

function writeImageResults(resultjson){
    // Handle the response from a search for related images - basically 4 or 5 images (if anything exists) related to the main search
    resp = JSON.parse(resultjson);
    results = resp['results'];
    
    if (results.length < 1){
        return;
    }
    
    resultcont = document.getElementById('imagecrosslink');
    
    
    h3 = document.createElement('h3');
    h3.innerHTML = "Images";
    resultcont.appendChild(h3);
    
    var e;
    for (var i=0;i<results.length;i++){
    
        if (window.seen.includes(results[i]['key'])){
            console.log(window.seen)
            console.log("Skipp " + results[i]['key'] )
            continue;
        }
    
        e = createImageNail(results[i]);

        resultcont.appendChild(e);
        window.seen.push(results[i]['key']);
    }
    
    var d = document.createElement('div');
    d.className = 'crosslink';
    var a = document.createElement('a');
    a.addEventListener('click', function(){  changeType('image'); });
    a.innerHTML="More results in images";
    d.appendChild(a);
    resultcont.appendChild(d);
    
}

function failedSearch(resultjson){
    console.log("Search failed. Uhoh");
    document.getElementById('loading').style.display = 'none';
    document.getElementById('resultcount').innerHTML = "Search Failed. Please try again";
    document.getElementById('resultinfo').style.display = 'block';
}

function createResultRow(res){
    r = document.createElement('div');
    r.className = "resultrow";
    
    tit = document.createElement('div');
    tit.className = "resulttitle";
    
    ico = document.createElement('div');
    ico.className = "ico " + res['content-type'].replace("/", "-");
    tit.appendChild(ico);

    
    
    link = document.createElement('a');
    link.setAttribute('href',res['key']);
    if (res['title'].length > 1) {
        txt = document.createTextNode(res['title']);
    }else{
        txt = document.createTextNode(res['key']);
    }
    link.appendChild(txt);
    tit.appendChild(link);    

    r.appendChild(tit);
    
    url = document.createElement('div');
    url.className = "resulturl";
    
    link = document.createElement('a');
    link.setAttribute('href',res['key']);
    txt = document.createTextNode(res['key']);
    link.appendChild(txt);
    url.appendChild(link);
    r.appendChild(url);
    
    
    snip = document.createElement('div');
    snip.className = "resultsnippet";    
    link = document.createElement('a');
    link.setAttribute('href',res['key']);
    snippara = document.createElement('span');
    snippara.className = 'tagset';
    
    if (res['keywords']){
        for (var i=0; i<res['keywords'].length; i++){
            // Construct and link to a search for the tag
            sp = document.createElement('a');
            sp.href="?q=" + res['keywords'][i] + " matchtype:tag mode:exact";
            sp.className = 'tag'
            txt = document.createTextNode(res['keywords'][i]);
            sp.appendChild(txt);
            snippara.appendChild(sp);
        }
    }
    
    if (res['implied-tag']){
        sp = document.createElement('a');
        sp.href="?q=" + res['implied-tag'] + " matchtype:tag mode:exact";
        sp.className = 'tag'
        txt = document.createTextNode(res['implied-tag'] + " (implied)");
        sp.appendChild(txt);
        snippara.appendChild(sp);        
    }
    
    link.appendChild(snippara);
    snip.appendChild(link);
    r.appendChild(snip);
    
    
    rd = document.createElement('div');
    rd.className = "resultdates";
    
    s1 = document.createElement('span');
    s1.className = "lastcheck";
    s1.innerHTML="Last Checked: "
    sd1 = document.createElement('span');
    sd1.className = "date";
    sd1.appendChild(document.createTextNode(res['valid-at']));
    
    s1.appendChild(sd1);
    rd.appendChild(s1);
    
    
    if (res['last-mod'] && res['last-mod'].length > 0){
        if (res['last-mod'] == "FFFFFF"){
            lm = "Unknown";
        }else{
            // Convert the Last mod format date
            d = new Date(res['last-mod']);
            lm = d.getFullYear() + "-" + (d.getMonth() + 1) + "-" + d.getDate();
        }
        s2 = document.createElement('span');
        s2.className = "lastmod";
        s2.innerHTML="Last Mod: "
        sd2 = document.createElement('span');
        sd2.className = "date";
        
        sd2.appendChild(document.createTextNode(lm));
        
        s2.appendChild(sd2);
        rd.appendChild(s2);
    }
    r.appendChild(rd);
    
    
    fs = document.createElement('div');
    fs.className = "fileSize";
    fs.innerText = convertBytes(res['bytes']);
    r.appendChild(fs);

    rd0 = document.createElement('div');
    rd0.className = "matchtype";
    
    s1 = document.createElement('span');
    s1.className = "matchtype";
    s1.innerHTML="Match Type: "
    sd1 = document.createElement('span');
    sd1.className = "typeinfo";
    
    if (res['tagtype'] == "implied"){
        sd1.appendChild(document.createTextNode(res['matchtype'] + " (implied)"));    
    }else{
        sd1.appendChild(document.createTextNode(res['matchtype']));   
    }
    
    s1.appendChild(sd1);
    rd0.appendChild(s1);    
    r.appendChild(rd0);    
    
    return r;
}

/** Convert a value in bytes to an appropriate unit
 * 
 * 
 */
function convertBytes(b){
    if (!b){
        return '';
    }
    
    units = [
      [1099511627776, 'TiB'],
      [1073741824, 'GiB'],
      [1048576, 'MiB'],
      [1024, 'KiB'],
    ]
    
    for (var i=0; i<units.length; i++){
        if (b > units[i][0]){
          v = (b / units[i][0]).toFixed(2);
          return v.toString() + " " + units[i][1];
        }
    }
    
    // Didn't match anything, so must be talking bytes
    return b.toString() + " bytes";
}

function createImageNail(res){
    console.log("Do thumb")
    r = document.createElement('div');    
    r.className = "imagewrap";
    
    a = document.createElement('a');
    a.setAttribute('href',res['key']);
    a.setAttribute('target','_blank');
    
    
    // Calculate the thumbnail path
    if (res['content-type'] != 'image/gif'){
        thumbsrc = "/thumbs/" + res['image-path'] + "/" + res['hashkey'] + ".jpg"
    }else{
        thumbsrc = "/thumbs/" + res['image-path'] + "/" + res['hashkey'] + ".gif"
    }
        
    // Extract the filename
    filename = res['key'].substring(res['key'].lastIndexOf("/")+1).split("?")[0];
    
    a.setAttribute('data-src',thumbsrc);
    a.setAttribute('data-domain',res['domain']);
    a.setAttribute("data-real-src",res['key']);
    a.setAttribute("data-filename",filename);
    a.setAttribute("data-filesize",convertBytes(res['bytes']));
    if ("embedded-in" in res){
        a.setAttribute("data-embedded-at",res['embedded-in']);
    }
    
    title = 'nil'
    
    if (res['title'] && res['title'].length > 0){
        title = res['title'];
    }
    
    a.setAttribute("data-title",title);
    
    a.addEventListener('click',showImagePreview);
    
    i = document.createElement('img');
    i.setAttribute("loading", "lazy");
    i.setAttribute('src',thumbsrc);
    i.setAttribute('data-src',thumbsrc);
    i.setAttribute('data-domain',res['domain']);
    i.setAttribute("data-real-src",res['key']);
    i.setAttribute("data-filename",filename);
    i.setAttribute("data-thumb-reachable","uk");
    i.setAttribute("data-title",title);
    i.setAttribute("data-filesize",convertBytes(res['bytes']));
    if ("embedded-in" in res){
        i.setAttribute("data-embedded-at",res['embedded-in']);
    }    
    i.addEventListener('error',switchtoorig);
    
    a.appendChild(i);
    
    r.appendChild(a);
    return r;
    
}

function switchtoorig(event){
    var ele = event.srcElement;
    ele.setAttribute('src',"/static/img-placeholder.png");
    ele.setAttribute("data-thumb-reachable","no");
    
    if (ele.parentNode.parentNode.className == "imagewrap"){
        ele.parentNode.parentNode.setAttribute('style','max-width: 5%');
    }
    
}


function showImagePreview(event){
    console.log("Build preview")
    event.preventDefault();
    var ele = event.srcElement;
    
    thumb = ele.getAttribute('data-src');
    domain = ele.getAttribute('data-domain');
    url = ele.getAttribute("data-real-src");
    filename = ele.getAttribute("data-filename");
    title = ele.getAttribute('data-title');
    embedded = ele.getAttribute('data-embedded-at');
    filesize = ele.getAttribute('data-filesize');
    
    srcreachable = ele.getAttribute('data-thumb-reachable');
    
    if (srcreachable == "no"){
        document.getElementById('previewimganchor').setAttribute('src','/static/img-placeholder.png');        
    }else{
        document.getElementById('previewimganchor').setAttribute('src',thumb);        
    }

    
    document.getElementById('previewimagelink').setAttribute('href',url);
    document.getElementById('imgpreviewfilesize').innerText = filesize;

    tit = document.getElementById('imgpreviewtitle');
    tit.innerHTML = '';
    
    if (title != "nil"){
        t = document.createTextNode(title);
    }else{
        t = document.createTextNode(filename);
    }
    tit.appendChild(t);
    
    tit = document.getElementById('imgpreviewdomain');
    tit.innerHTML = '';
    t = document.createTextNode(domain);
    tit.appendChild(t);
    tit.setAttribute('href',"https://"+domain);
    
    document.getElementById('imgpreviewfulllink').setAttribute('href',url);
    
    if (embedded && embedded.length > 1){
        document.getElementById('imgcontainingpage').setAttribute('href', embedded);
        document.getElementById('imgcontainingpage').style.display = 'inline';
    }else{
        document.getElementById('imgcontainingpage').style.display = 'none';
    }
    
    

    prev = ele.parentNode.parentNode.previousSibling;
    if (prev){
        window.previmg = prev.getElementsByTagName('img')[0];
        document.getElementById('prevbutton').style.visibility = 'visible';
    }else{
        document.getElementById('prevbutton').style.visibility = 'hidden';
        window.previmg = false;
    }

    next = ele.parentNode.parentNode.nextSibling;
    if (next){
        window.nextimg = next.getElementsByTagName('img')[0];
        document.getElementById('nextbutton').style.visibility = 'visible';
    }else{
        document.getElementById('nextbutton').style.visibility = 'hidden';
        window.nextimg = false;
    }
    
    document.getElementById('previewblock').style.display = 'block';
}


function closepreview(){
    document.getElementById('previewblock').style.display = 'none';
}


function previewnext(){
    if (window.nextimg){
        window.nextimg.click();
    }
}

function previewprev(){
    if (window.previmg){
        window.previmg.click();
    }
}

function relatedImagessearch(){
    // Used to retrieve a limited number of related images
    term = document.getElementById('searchterm').value;
    searchobj = {
        "term" : term,
        "type" : "IMAGE",
        "limit" : 8
        }
        
    fetchPage("/search/",searchobj,writeImageResults,failedSearch);
}

function toggleHelp(){
    var d = document.getElementById('helptext');
    
    if (d.style.display == "none"){
        d.style.display = "block";
    }else{
        collapseHelp();
    }
}

function collapseHelp(){
    document.getElementById('helptext').style.display = "none";    
}
