#!/usr/bin/env python3
#
# Delete entries for a URL
#
'''
Copyright (c) 2023 B Tasker

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

    Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

    Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

    Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
'''

import os
import sys

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

import config
import storage
    

if __name__ == "__main__":
    DB_PATH = config.getDBPath()
    storage.DB_PATH = DB_PATH
    prefix = False
    
    try:
        url = sys.argv[1]
        if sys.argv[1] == "-p":
            prefix = True
            url = sys.argv[2]
    except:
        print(f"Usage: {sys.argv[0]} [-p] [url]")
        sys.exit(1)
    

    if not prefix:
        # Delete the one item
        print(f"Deleting {url}")
        storage.deleteStoredItem(url)
    else:
        # We need to load the index so that we can identify
        # urls starting with the provided prefix and delete those
        storage.INDEX_ALL = True
        storage.loadIndex()
        
        for e in storage.INDEXES['ALL']:
            if e['u'].startswith(url):
                print(f"Deleting {e['u']}")
                storage.deleteStoredItem(e['u'])
                
